@@ -12,7 +12,8 @@
 //use ruma_identifiers::{RoomId, UserId};
 //use serde::{Serialize, Deserialize};
 use scanpw::scanpw;
-use crackle_lib::RoomId;
+use serde::{Serialize, Deserialize};
+use crackle_lib::{RoomId, UserId};
 use crossbeam_channel::{unbounded, TryRecvError};
 use crossterm::{
     event::{self, DisableMouseCapture, EnableMouseCapture, Event, KeyCode},
@@ -36,6 +37,13 @@ use unicode_width::UnicodeWidthStr;
 //use crossbeam_utils::atomic::AtomicCell;
 
 //static message_box_height: AtomicCell<u16> = AtomicCell::new(3); 
+#[derive(Serialize, Deserialize, Debug, Clone)]
+struct CrackleConfig {
+    matrix_url: String, 
+    auth_token: String,
+    username: String,
+    room: String,
+}
 
 enum Focus {
     MessageInput,
@@ -59,8 +67,10 @@ struct App {
     host: String,
     server_name: String,
     user_name: String,
-    password: String,
+    password: Option<String>,
     room: String,
+    //using_config: bool,
+    auth_token: Option<String>,
     //message_box_height: u16,
 }
 /*
@@ -78,7 +88,7 @@ impl Default for App {
 
 
 impl App {
-    pub async fn new(home_server_url: impl Into<String>, user_name: String,  password: String, room: String) -> Self {
+    pub async fn new(home_server_url: impl Into<String>, user_name: String,  password: Option<String>, room: String, auth_token: Option<String>) -> Self {
         let mut hs_url = home_server_url.into();
         //let mut host: String;
         if !hs_url.starts_with("http://") && !hs_url.starts_with("https://") {
@@ -86,16 +96,20 @@ impl App {
         }
         let hostparts = hs_url.split_once("://").unwrap();
         let server_name = hostparts.1.clone().split(":").map(|z| z.to_string()).collect::<Vec<String>>()[0].clone();
-        
+        let username = match UserId::try_from(user_name.clone()) {
+            Ok(user_id) => user_id.localpart().to_string(),
+            Err(_) => user_name.clone(),
+        };
         Self {
             input: String::new(),
             focus: Focus::Watching,
             messages: Vec::new(),
             host: hs_url,
             server_name: server_name,
-            user_name: user_name,
+            user_name: username,
             password: password,
             room: room,
+            auth_token: auth_token, 
             //message_box_height: 3,
             //client: ruma_client::Client::builder().homeserver_url(home_server.into()).build::<ruma_client::http_client::Reqwest>().await.unwrap(),
         }
@@ -104,16 +118,32 @@ impl App {
 
 #[tokio::main]
 async fn main() -> Result<(), Box<dyn Error>> {
-    let mut host = String::new();
-    println!("Matrix URL: ");
-    std::io::stdin().read_line(&mut host).unwrap();
-    let mut username = String::new();
-    println!("Username: ");
-    std::io::stdin().read_line(&mut username).unwrap();
-    let password = scanpw!("Password: ").trim().to_string();
-    let mut room = String::new();
-    println!("\nRoom: ");
-    std::io::stdin().read_line(&mut room).unwrap();
+    //let app: App;
+    let mut host: String;
+    let mut username: String;
+    let password: Option<String>;
+    let auth_token: Option<String>;
+    let mut room: String;
+    if std::path::Path::new("./config.yaml").exists() {
+        let config: CrackleConfig = serde_yaml::from_str(&std::fs::read_to_string("./config.yaml").unwrap()).unwrap();
+        host = config.matrix_url;
+        username = config.username;
+        auth_token = Some(config.auth_token);
+        room = config.room;
+        password = None;
+    } else {
+        host = String::new();
+        println!("Matrix URL: ");
+        std::io::stdin().read_line(&mut host).unwrap();
+        username = String::new();
+        println!("Username: ");
+        std::io::stdin().read_line(&mut username).unwrap();
+        password = Some(scanpw!("Password: ").trim().to_string());
+        room = String::new();
+        println!("\nRoom: ");
+        std::io::stdin().read_line(&mut room).unwrap();
+        auth_token = None;
+    }
     // setup terminal
     enable_raw_mode()?;
     let mut stdout = io::stdout();
@@ -122,7 +152,7 @@ async fn main() -> Result<(), Box<dyn Error>> {
     let mut terminal = Terminal::new(backend)?;
 
     // create app and run it
-    let app = App::new(host.trim().to_string(), username.trim().to_string(), password, room.trim().to_string()).await;
+    let app = App::new(host.trim().to_string(), username.trim().to_string(), password, room.trim().to_string(), auth_token).await;
     let res = run_app(&mut terminal, app).await;
 
     // restore terminal
@@ -179,7 +209,12 @@ async fn run_app<B: Backend>(terminal: &mut Terminal<B>, mut app: App) -> io::Re
     //let distrotube_room_id = client.send_request(ruma_client_api::alias::get_alias::v3::Request::new(ruma_identifiers::room_alias_id!("#distrotube:matrix.org"))).await.unwrap().room_id;
     //println!("'{}'", app.server_name.clone());
     let mut client = crackle_lib::Client::new(app.host.clone(), app.user_name.clone(), app.server_name.clone());
-    client.login(app.password.clone());
+    if let Some(pass) = app.password.clone() {
+        client.login(pass);
+    } else {
+        client.set_auth_token(app.auth_token.as_ref().unwrap().clone());
+    }
+    
     let me = client.get_me();
     let (tx1, rx1) = unbounded::<SyncMessage>();
     let tx1_clone = tx1.clone();
